{% extends "catalogue/browse.html" %}

{% load i18n %}

{% block title %}
    {{ category.name }} | {{ block.super }}
{% endblock %}

{% block description %}
    {{ category.description|default:""|striptags }}
{% endblock %}

{% block headertext %}{{ category.name }}{% endblock %}

{% block breadcrumbs %}
    <ul class="breadcrumb">
        <li>
            <a href="{{ homepage_url }}">{% trans "Home" %}</a>
            <span class="divider">/</span>
        </li>
        {% for category in category.get_ancestors %}
            <li>
                <a href="{{ category.get_absolute_url }}">{{ category.name }}</a>
                <span class="divider">/</span>
            </li>
        {% endfor %}
        <li class="active">{{ category.name }}</li>
    </ul>
{% endblock breadcrumbs %}

{% block content %}
    {% if category.description %}
        <div class="row-fluid">
            <div class="span9"><p>{{ category.description|safe }}</p></div>
            {% if category.image %}
                <div class="span3"><img src="{{ category.image.url }}" alt="{{ category.name }}" /></div>
            {% endif %}
        </div>
    {% endif %}
    {{ block.super }}
{% endblock %}
